/**
 * @file mutex.cpp
 *
 * @author Jan Hadrava
 * @ingroup iopccommon
 */

#include <pthread.h>
#include <errno.h>

#include "mutex.h"
#include "defs.h"
#include "trace.h"

namespace iopc {
Mutex::Mutex()
	throw (IopcNoMemoryException, IopcRsrcLackException)
{
	switch (pthread_mutex_init(&mutex, NULL)) {
	case 0:
		return;
	break;
	case ENOMEM:
		throw IopcNoMemoryException();
	break;
	case EAGAIN:
	case EPERM:
		throw IopcRsrcLackException();
	break;
	default:
		IOPC_UNREACHABLE;
	break;
	}
}

Mutex::~Mutex()
	throw ()
{
	int res = pthread_mutex_destroy(&mutex);
	if (res != 0) {
		IOPC_UNREACHABLE;
	}
}

void Mutex::lock()
	throw (IopcDeadlockException)
{
	int res;
	switch (res = pthread_mutex_lock(&mutex)) {
	case 0:
		return;
	break;
	case EDEADLK:
		throw IopcDeadlockException();
	break;
	default:
		IOPC_UNREACHABLE;
	break;
	}
}

void Mutex::unlock()
	throw (IopcIncorrectUsageException)
{
	switch (pthread_mutex_unlock(&mutex)) {
	case 0:
		return;
	break;
	case EPERM:
		throw IopcIncorrectUsageException();
	break;
	default:
		IOPC_UNREACHABLE;
	break;
	}
}

}
