/**
 * @file basicConnection.h
 *
 * @author Michal Kopecky
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>)
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef BASICCONNECTION_H
#define BASICCONNECTION_H

#include "iopccommon/lock.h"
#include "iopccommon/defsPlatform.h"
#include "database.h"
#include "cursor.h"
#include "connectionImpl.h"
#include "connection.h"
#include "basicDatabase.h"
#include <string>
#include <set>

namespace iopc
{

class Database;
class Cursor;

/**
 * @brief The driver-independent part of the @a Connection implementation.
 *
 * @ingroup iopcdb
 */
class DLL_EXTERNAL BasicConnection : public Connection
{

public:
	typedef std::set<Cursor*> CursorSet;
	typedef CursorSet::iterator CursorSetIterator;

private:
	BasicDatabase& database;		//the database this connection is
	bool autoCommit;
	bool opened;

	static CursorSet allCursors;
	CursorSet cursors;
	void addCursor(Cursor* cur);
	void removeCursor(Cursor* cur);

	ConnectionImpl* implementation;

	Lock lock;
	static Lock allLock;

public:

	virtual ~BasicConnection();

	virtual Cursor* sql(const std::string& name);

	virtual void returnCursor(Cursor* cur);

	virtual void closeAllCursors();

	virtual void sqlNonQuery(const std::string& name);

	virtual void commit();

	virtual void rollback();

	virtual void savepoint(const std::string& name);

	virtual void rollbackToSavepoint(const std::string& name);

	virtual void open();

	virtual void close();

	virtual void disconnect();

	virtual void abort();

	virtual void setAutoCommit(bool ac);

	virtual bool isOpened();

	virtual Connection& operator << (
	    const std::string& command
	);


	virtual bool operator== (const Connection& dbConn) const;

	virtual bool operator!= (const Connection& dbConn) const;

	inline Database& getDatabase() { return database; }

	inline Driver& getDriver() { return database.getDriver(); }

	inline ConnectionImpl* getImplementation() { return implementation; }

	BasicConnection(ConnectionImpl* implementation, BasicDatabase& database); /*, Cache *cache*/

	Connection* getBasicConnection();

};


}

#endif

