/**
 * @file basicDatabase.h
 *
 * @author Michal Kopecky
 * @author Jan Hadrava
 * @author Josef Troch
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef BASICDATABASE_H
#define BASICDATABASE_H

#include <fstream>
#include <set>
#include "iopccommon/defsPlatform.h"
#include "database.h"
#include "iopccommon/mutex.h"


namespace iopc {

/**
 * @brief The driver-independent part of the @a Database implementation.
 *
 * @ingroup iopcdb
 */
class DLL_EXTERNAL BasicDatabase : public Database {

private:
	typedef std::set<Connection*> ConnectionSet;
	typedef ConnectionSet::iterator ConnectionSetIterator;
	//std::string connectionString;
	static ConnectionSet allConnections;
	ConnectionSet connections;

	void addConnection(Connection* con);
	void removeConnection(Connection* con);

	DatabaseImpl* implementation;

	Lock lock;
	static Lock allLock;
	Driver& driver;


public:
	BasicDatabase(DatabaseImpl* implementation, Driver& driver);
	virtual ~BasicDatabase();

	virtual void commit();
	virtual void rollback();
	virtual Connection* getConnection(const std::string& connStr);
	virtual void returnConnection(Connection* connection);
	virtual void disconnect();

	inline ConnectionSet getConnections() { return connections; }

	inline static ConnectionSet getAllConnections() { return allConnections; }

	inline DatabaseImpl* getImplementation() { return implementation; }

	inline Driver& getDriver() { return driver; }

	virtual Database* getBasicDatabase();

}; // class


} // namespace

#endif
