/**
 * @file dbFeature.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */


#ifndef DBFEATURE_H_
#define DBFEATURE_H_

#include <string>

namespace iopc {

/**
 * @brief Base class of all driver features
 *
 * Driver features reflect differences between database systems.
 * Different database systems provide different features or services.
 * Modules of the IOPC 2 library define interfaces which derive from @a DbFeature
 * (representing the features) and database driver creators optionally implement
 * them to their drivers or group them in driver extensions. The library then
 * asks the drivers what features they offer. Based on the answers it enables
 * or disables some of its functionalities provided.
 *
 * @ingroup iopcdb
 *
 */
class DbFeature
{
public:
	/**
	 * @brief Returns name of the database feature
	 *
	 * Database features must have unique names across the library
	 */
	virtual const std::string& getNameInst() =0;
	virtual ~DbFeature() {};
};


}

#endif /*DBFEATURE_H_*/
