/**
 * @file simpleCacheSelector.cpp
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */

#include "simpleCacheSelector.h"

namespace iopc {

SimpleCacheSelector::SimpleCacheSelector()
: cacheMap()
{
	lastIter = cacheMap.end();
}

SimpleCacheSelector::~SimpleCacheSelector()
{
	for (Iter iter = cacheMap.begin(); iter != cacheMap.end(); iter++) {
		CacheRecord *cr = iter->second;
		if (cr->isMine) {
			cr->cache->setConnection(NULL);
			delete cr->cache;
		}
		delete cr;
	}
	cacheMap.clear();
}

ExtendedCache* SimpleCacheSelector::firstCache()
{
	lastIter = cacheMap.begin();
	return lastIter == cacheMap.end() ? NULL : lastIter->second->cache;
}

ExtendedCache* SimpleCacheSelector::nextCache()
{
	lastIter++;
	return lastIter == cacheMap.end() ? NULL : lastIter->second->cache;
}

ExtendedCache* SimpleCacheSelector::selectCache(const PersIdentification& persId)
{
	Iter iter = cacheMap.find(&persId.getType());	// use cache according to the prototype
	if (iter != cacheMap.end()) {
		return iter->second->cache;
	}
	/*iter = cacheMap.find(Class[persId.proto->BaseClassName()]);// use cache according to the base class
	if (iter != cacheMap.end()) {
		return iter->second->cache;
	}*/
	iter = cacheMap.find(NULL);		// use default cache
	if (iter != cacheMap.end()) {
		return iter->second->cache;
	}
	return NULL;	// this will probably have grave consequences */
}


void SimpleCacheSelector::setCache(const Type& type, ExtendedCache* cache, bool isMine)
{
	setCacheInternal(&type, cache, isMine);
}

void SimpleCacheSelector::setCache(ExtendedCache* cache, bool isMine)
{
	setCacheInternal(NULL, cache, isMine);
}

void SimpleCacheSelector::setCacheInternal(const Type* type, ExtendedCache* cache, bool isMine)
{
	Iter iter = cacheMap.find(type);
	if (iter != cacheMap.end()) {
		CacheRecord *cr = iter->second;
		if (cr->isMine) {
			delete cr->cache;
		}
		if (cache == NULL) {
			cacheMap.erase(iter);
			delete cr;
			return;
		}
		cr->cache = cache;
		cr->isMine = isMine;
	} else {
		if (cache == NULL) {
			return;
		}
		CacheRecord *cr = new CacheRecord();
		cr->cache = cache;
		cr->isMine = isMine;
		cacheMap[type] = cr;
	}
}

}/*namespace iopc*/
