/**
 * @file simpleStrategySelector.cpp
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */

#include "simpleStrategySelector.h"

namespace iopc {

SimpleStrategySelector::SimpleStrategySelector(void *wakeUpData, WakeUpMntrFunction wakeUpFunction, CostFunction costFunction)
: stratMap(), wakeUpData(wakeUpData), wakeUpFunction(wakeUpFunction), costFunction(costFunction)
{
	defaultStrategy = Strategy(true, false, USDeferred, RSCache,
		wakeUpData, wakeUpFunction, costFunction);
}

SimpleStrategySelector::~SimpleStrategySelector()
{
	for (Iter iter = stratMap.begin(); iter != stratMap.end(); iter++) {
		delete iter->second;
	}
	stratMap.clear();
}

void SimpleStrategySelector::offline()
{
	for (Iter iter = stratMap.begin(); iter != stratMap.end(); iter++) {
		iter->second->offline();
	}
	defaultStrategy.offline();
}

const Strategy& SimpleStrategySelector::select() const
{
	Iter iter = stratMap.find(NULL);		// use default strategy
	if (iter != stratMap.end()) {
		return *iter->second;
	}
	return defaultStrategy;
}

const Strategy& SimpleStrategySelector::select(const PersIdentification& persId) const
{
	Iter iter = stratMap.find(&persId.getType());	// use strategy according to the prototype
	if (iter != stratMap.end()) {
		return *iter->second;
	}
	// TODO: treba z metadat...
	/*iter = stratMap.find(Class[persId.proto->BaseClassName()]);// use strategy according to the base class
	if (iter != stratMap.end()) {
		return *iter->second;
	}*/
	iter = stratMap.find(NULL);		// use default strategy
	if (iter != stratMap.end()) {
		return *iter->second;
	}
	return defaultStrategy;
}

void SimpleStrategySelector::setStrategy(const Type& type, bool loadWait, bool loadLocking, UpdatingStrategy uStrat, ReadingStrategy rStrat)
{
	setStrategyInternal(&type, loadWait, loadLocking, uStrat, rStrat);
}

void SimpleStrategySelector::setStrategy(bool loadWait, bool loadLocking, UpdatingStrategy uStrat, ReadingStrategy rStrat)
{
	setStrategyInternal(NULL, loadWait, loadLocking, uStrat, rStrat);
}

void SimpleStrategySelector::setStrategyInternal(const Type* type, bool loadWait, bool loadLocking, UpdatingStrategy uStrat, ReadingStrategy rStrat)
{
	Iter iter = stratMap.find(type);
	if (iter == stratMap.end()) {
		Strategy *strategy = new Strategy(true, false, USDeferred, RSCache, wakeUpData, wakeUpFunction, costFunction);
		stratMap[type] = strategy;
		iter = stratMap.find(type);
	}
	IOPC_ASSERT(iter != stratMap.end());
	iter->second->set(loadWait, loadLocking, uStrat, rStrat);
}

}/*namespace iopc*/
