/**
 * @file databaseObject.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef DATABASEOBJECT_H_
#define DATABASEOBJECT_H_

#include "iopcmeta/enhancedString.h"
#include "iopcmeta/enhancedNumber.h"
#include "iopcmeta/object.h"
#include "iopccommon/defs.h"
#include "iopcdb/connection.h"
#include "iopcmeta/typeDesc.h"
#include "iopcmeta/metaSupport.h"
#include <map>
#include <string>

namespace iopc {

//typedef std::map<std::string, Object> KeyValuesType;

/**
 * @brief Provides object persistence for its descendants
 *
 * @a DatabaseObject descendants are persistent classes that describe identity of their instances
 * as a list of key attributes - @a KeyValues. Database objects do not necessarily need to have
 * any identity defined as they may represent for example aggregate query results.
 * @a DatabaseObject -s with the key attributes defined can be mapped into database tables.
 * They even support vertical and horizontal database mapping. However, the subclasses have to share
 * the same set of key attributes. This enables inheritance, but polymorphism is somewhat limited
 * as IOPC 2 is not capable of recovering the effective type of an object from a set of key attribute values.
 * IOPC 2 doesn't generate structures for classname resolution based on the key values,
 * so when loading an object identified by its key values, the type of the loaded object must be specified.
 * That is also the reason why filtered mapping is not supported for non-OID objects.
 *
 * @ingroup iopclib
 */
class DLL_EXTERNAL DatabaseObject : public Object {

private:
	static void iopcInit(iopc::Type& t) {}

	bool initializedAfterLoad;
protected:
	void onIopcCreate();

	/**
	 * @brief This method is invoked after an implicit object instance is created
	 * using @a DbPtr<T>().
	 *
	 * This method sould not be overriden by users.
	 */
	virtual void onCreate() {};

	//virtual void onLoad() {};
	// TODO: create remaining events.. onCacheDelete etc..., public:

public:

	DatabaseObject();

	virtual ~DatabaseObject() {};

	inline virtual bool isOidObject() { return false; }

	friend class DbPtrBase; // onIopcCreate();

	friend class TypeDesc<DatabaseObject>;
};

// TypeDesc for OidType

}

// generate TypeDesc
META_BLOCK1(iopc::DatabaseObject)
META_PARENT1(iopc::Object)
META_BLOCK2(iopc::DatabaseObject)
META_BLOCK3(iopc::DatabaseObject, DatabaseObject, iopc)
META_INIT(iopc::DatabaseObject)
META_BLOCK4

#endif /*DATABASEOBJECT_H_*/
