/**
 * @file freeQuery.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef FREEQUERY_H_
#define FREEQUERY_H_

#include "query.h"

namespace iopc {


class Type;
class Connection;
class Driver;

/**
 * @brief A query implementation that more complex queries to be created than the @a SimpleQuery class
 *
 * This class allows developers to specify everything after the FROM keyword. There are absolutely
 * no restrictions on joins, subselects or other SQL language constructs.
 *
 * @ingroup iopclib
 */
class DLL_EXTERNAL FreeQuery : public Query {

private:
	std::string query;
	std::string prefix;
	bool useViewColumns;

public:

	/**
	 * @param queryStr A query string that will be rendered after the FROM keyword in the resulting SELECT statement.
	 * The string is substituted according to rules described in @a Query::substituteQuery()
	 * @param prefix A prefix that will be rendered before each column name in the SELECT column list.
	 * Example: If a query prefix is "x", the generated statement will be SELECT x.col1, x.col2 FROM...
	 * @param useViewColumns Determines whether to render [db.viewColumn] or [db.column] for column names
	 */
	FreeQuery(std::string queryStr, const std::string& prefix = "", bool useViewColumns = true) : query(queryStr), prefix(prefix), useViewColumns(useViewColumns) {};
	//FreeQuery(std::string queryStr, const std::string& prefix) : query(queryStr), prefix(prefix), useViewColumns(true) {};
	//FreeQuery(std::string queryStr, bool useViewColumns) : query(queryStr), prefix(""), useViewColumns(useViewColumns) {};
	FreeQuery(const FreeQuery& query) : query(query.query), prefix(query.prefix), useViewColumns(useViewColumns) {};

	virtual std::string getSql(Driver& drv, const Type& type) const;

protected:
	virtual Query* clone() const;


};








}


#endif /*FREEQUERY_H_*/
